# Model planowania dystrybucji dla zadanego planu produkcji (przezbroje) w ramach VMI
#
#======================================================================
param Model symbolic := "VMI_P_D";

param St_wykorzystania >= 0, <= 1;
param FileCSV symbolic :=
	"VMI_P_D_0"&substr(St_wykorzystania,3)&".csv";
#======================================================================
param Liczba_dni > 0;
set Dni := 1..Liczba_dni;

set Produkty;
set Samochody;
set Platformy;

set Okazje;

#======================================================================

param Koszt_mag 		>= 0;
param Koszt_przezbr 	>= 0;
param Koszt_str_popytu 	>= 0;

param Czas_wyk						{Produkty} >= 0;
param Pocz_gotowosc				{Produkty} binary;

param Pocz_zapas_dostawcy	{Produkty} >= 0;
param Pocz_zapas_odbiorcy	{Platformy, Produkty} >= 0;
param Zapas_bezp 				{Platformy, Produkty} >= 0;
param Popyt 						{Platformy, Produkty, Dni} >= 0;

table T_Pocz_zapas_dostawcy	IN "CSV" "Pocz_zapas_dostawcy.csv":
	[Produkt], Pocz_zapas_dostawcy~Pocz_zapas_dostawcy;

table T_Zapas_bezp IN "CSV" "Zapas_bezp.csv":
	[Platforma, Produkt], Zapas_bezp~Zapas_bezp	;

table T_Pocz_zapas_odbiorcy IN "CSV" "Pocz_zapas_odbiorcy.csv":
	[Platforma, Produkt], Pocz_zapas_odbiorcy~Pocz_zapas_odbiorcy	;

table T_Popyt IN "CSV" "Popyt.csv":
	[Platforma, Produkt, Dzien],  Popyt~Popyt;

param Koszt_transp	{Samochody} >= 0;
param Ladownosc 		{Samochody} >= 0;
param Czas_transp		{Samochody} >= 0;
param min_Czas_transp := min {s in Samochody} Czas_transp[s];

param Cel_O				{Okazje} symbolic in Platformy;
param Termin_O			{Okazje} >= 0;
param Koszt_transp_O	{Okazje} >= 0;
param Ladownosc_O 	{Okazje} >= 0;
param Czas_transp_O	{Okazje} >= 0;

param Obciazenie :=
	sum {j in Produkty} Czas_wyk[j] *
		(sum {p in Platformy} (sum {t in Dni} Popyt[p, j, t]	 - Pocz_zapas_odbiorcy[p, j])
		- Pocz_zapas_dostawcy[j]);

param Czas_pracy := Obciazenie / (Liczba_dni-min_Czas_transp) / St_wykorzystania;

#======================================================================
param Produkcja 	{Produkty, Dni} >= 0;
param Przezbrojenie	{Produkty, Dni} binary;
param Gotowosc 	{Produkty, Dni union {0}} binary;
var Zapas_dostawcy					 {j in Produkty, Dni union {0}} >= 0;

var Transport  			{Platformy, Samochody, Dni} binary;
var Wielkosc_transp 	{Platformy, Samochody, Dni, Produkty} >= 0;
var Zapas_odbiorcy	{p in Platformy, j in Produkty, Dni union {0}} >= Zapas_bezp[p,j];
var Stracony_popyt	{p in Platformy, j in Produkty, Dni union {0}} >= 0;

var Transport_O 			{o in Okazje} binary;
var Wielkosc_transp_O {o in Okazje, j in Produkty} >= 0, <= Ladownosc_O[o];

table przezbrojenia IN "CSV" "przezbrojenie.csv":
	[Produkt, Dzien], Przezbrojenie~Przezbrojenie;

table gotowosc IN "CSV" "gotowosc.csv":
	[Produkt, Dzien], Gotowosc~Gotowosc;

table produkcja IN "CSV" "produkcja.csv":
	[Produkt, Dzien], Produkcja~Produkcja;

var KosztyPrzezbrajania;
var KosztyZapasowDostawcy;
var KosztyTransportu;
var KosztyZapasowWDrodze;
var KosztyZapasowOdbiorcy;
var KosztyStraconegoPopytu;
var KosztyLaczne;

#======================================================================

minimize FC_KosztyLaczne:
					KosztyLaczne;

#======================================================================
subject to

	Ogr_KosztyLaczne:
		KosztyLaczne =
				KosztyPrzezbrajania
			+	KosztyZapasowDostawcy
			+	KosztyTransportu
			+	KosztyZapasowWDrodze
			+	KosztyZapasowOdbiorcy
			+	KosztyStraconegoPopytu;

	Ogr_KosztyPrzezbrajania:
		KosztyPrzezbrajania =
			sum {j in Produkty, t in Dni}
				Koszt_przezbr * Przezbrojenie[j, t];

	Ogr_KosztyZapasowDostawcy:
		KosztyZapasowDostawcy =
			sum {j in Produkty, t in Dni}
				Koszt_mag * Zapas_dostawcy[j, t];

	Ogr_KosztyTransportu:
			KosztyTransportu =
				sum {p in Platformy, s in Samochody, t in Dni}
					Koszt_transp[s] * Transport[p, s, t];

	Ogr_KosztyZapasowWDrodze:
		KosztyZapasowWDrodze =
			sum {p in Platformy, s in Samochody, t in Dni, j in Produkty}
				Koszt_mag * Czas_transp[s] * Wielkosc_transp[p, s, t, j];

	Ogr_KosztyZapasowOdbiorcy:
		KosztyZapasowOdbiorcy =
			sum {p in Platformy, j in Produkty, t in Dni}
				Koszt_mag * Zapas_odbiorcy[p, j, t];

	Ogr_KosztyStraconegoPopytu:
		KosztyStraconegoPopytu =
			sum {p in Platformy, j in Produkty, t in Dni}
				Koszt_str_popytu * Stracony_popyt[p, j, t]	;

	#------------------------------------------------------------------------------------------------------------------------
	# Produkcja
	#------------------------------------------------------------------------------------------------------------------------

	Ogr_Pocz_zapas_dostawcy {j in Produkty}:
		Zapas_dostawcy[j, 0] = Pocz_zapas_dostawcy[j];

	Ogr_Zdolnosc_prod {t in Dni}:
		sum {j in Produkty} Czas_wyk[j] * Produkcja[j, t] <= Czas_pracy;

	Ogr_Gotowosc {j in Produkty, t in Dni}:
		Czas_wyk[j] * Produkcja[j, t] <= Czas_pracy * (Gotowosc[j, t-1] + Gotowosc[j, t]);

	Ogr_Jednoznaczna_gotowosc {t in Dni}:
		sum {j in Produkty} Gotowosc[j, t] <= 1;

	Ogr_Stan_pocz {j in Produkty}:
		Gotowosc[j, 0] = Pocz_gotowosc[j];

	Ogr_Przezbrojenie {j in Produkty, t in Dni}:
		Gotowosc[j, t] - Gotowosc[j, t-1] <= Przezbrojenie[j, t];

	Ogr_Bilans_zapasu_dostwacy {j in Produkty, t in Dni}:
		Zapas_dostawcy[j, t-1] + Produkcja[j, t] =
		Zapas_dostawcy[j, t   ] 
		+ sum {p in  Platformy, s in Samochody}
				Wielkosc_transp[p, s, t, j]
		+ sum {o in Okazje: t = Termin_O[o]}
				Wielkosc_transp_O[o, j];

	NO_Przezbrojenie1 {j in Produkty, t in Dni}:
		(1 - Gotowosc[j, t-1]) >= Przezbrojenie[j, t];

	NO_Przezbrojenie2 {j in Produkty, t in Dni}:
		Gotowosc[j, t] >= Przezbrojenie[j, t];

	#~ NO_Zapas_dostwacy {j in Produkty, t in Dni, p in {0} union Dni:
						#~ t + min_Czas_transp  + p <= Liczba_dni and p <= 5}:
		#~ Zapas_dostawcy[j, t-1] >= sum {s in t..t+p}
				#~ sum {q in Platformy} Popyt[q, j, s+min_Czas_transp ]
					#~ * (1 - Gotowosc[j, t-1] - sum {r in t..s} Przezbrojenie[j, r]);

	#------------------------------------------------------------------------------------------------------------------------
	# Dystrybucja
	#------------------------------------------------------------------------------------------------------------------------

	Ogr_Pocz_zapas_odbiorcy {p in Platformy, j in Produkty}:
		Zapas_odbiorcy[p, j, 0] = Pocz_zapas_odbiorcy[p, j];

	Ogr_Bilans_zapasu_odbiorcy {p in Platformy, j in Produkty, t in Dni}:
		Zapas_odbiorcy[p, j, t-1]
		+ sum {s in Samochody: t-Czas_transp[s] >= 1}
			Wielkosc_transp[p, s, t-Czas_transp[s], j]
		+ sum {o in Okazje: p = Cel_O[o] and t - Czas_transp_O[o]=Termin_O[o]}
				Wielkosc_transp_O[o, j]
		= Zapas_odbiorcy[p, j, t] + Popyt[p, j, t] -	Stracony_popyt[p, j, t];

	Ogr_Transport_oraz_ladownosc {p in Platformy, s in Samochody, t in Dni}:
		sum {j in Produkty} Wielkosc_transp[p, s, t, j]
		<= Ladownosc[s] * Transport[p, s, t];

	Ogr_Transport_oraz_ladownosc_O {o in Okazje}:
		sum {j in Produkty} Wielkosc_transp_O[o, j]
		<= Ladownosc_O[o] * Transport_O[o];

solve;

	printf "Platforma;Samochod;Dzien;Produkt;Zmienna;Wartosc" > FileCSV;

	printf "\n;;;;%s;%s", 'Model', Model >> FileCSV;
	printf "\n;;;;%s;%s", 'Data', time2str(gmtime()+2*3600,'%F %H:%M') >> FileCSV;

	printf "\n;;;;%s;%3.1f", 'KosztyLaczne', KosztyLaczne  >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'KosztyPrzezbrajania', KosztyPrzezbrajania >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'KosztyZapasowDostawcy', KosztyZapasowDostawcy  >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'KosztyTransportu', KosztyTransportu >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'KosztyZapasowWDrodze', KosztyZapasowWDrodze >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'KosztyZapasowOdbiorcy', KosztyZapasowOdbiorcy >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'KosztyStraconegoPopytu', KosztyStraconegoPopytu >> FileCSV;

	printf "\n;;;;%s;%d", 'Liczba_dni', Liczba_dni >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'Czas_pracy', Czas_pracy >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'Koszt_mag', Koszt_mag  >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'Koszt_przezbr', Koszt_przezbr  >> FileCSV;
	printf "\n;;;;%s;%3.1f", 'Koszt_str_popytu', Koszt_str_popytu >> FileCSV;

	for {j in Produkty} {
		printf "\n;;;%s;%s;%3.1f", j, 'Pocz_zapas_dostawcy', Pocz_zapas_dostawcy[j] >> FileCSV;
		printf "\n;;;%s;%s;%3.1f", j, 'Czas_wyk', Czas_wyk[j] >> FileCSV;
	}

	for {p in Platformy, j in Produkty} {
		printf "\n%s;;;%s;%s;%3.1f", p, j, 'Pocz_zapas_odbiorcy', Pocz_zapas_odbiorcy[p, j] >> FileCSV;
		printf "\n%s;;;%s;%s;%3.1f", p, j, 'Zapas_bezp', Zapas_bezp[p, j] >> FileCSV;
	}

	for {p in Platformy, t in Dni, j in Produkty} {
		printf "\n%s;;%s;%s;%s;%3.1f", p, t, j, 'Popyt ', Popyt[p, j, t] >> FileCSV;
		printf "\n%s;;%s;%s;%s;%3.1f", p, t, j, 'Stracony_popyt', Stracony_popyt[p, j, t] >> FileCSV;
	}

	for {s in Samochody} {
		printf "\n;%s;;;%s;%3.1f", s, 'Czas_transp', Czas_transp[s] >> FileCSV;
		printf "\n;%s;;;%s;%3.1f", s, 'Koszt_transp', Koszt_transp[s] >> FileCSV;
		printf "\n;%s;;;%s;%3.1f", s, 'Ladownosc', Ladownosc[s] >> FileCSV;
	}

	for {j in Produkty, t in Dni} {
		printf "\n;;%d;%s;%s;%3.1f", t, j, 'Produkcja', Produkcja[j, t] >> FileCSV;
		printf "\n;;%d;%s;%s;%3.1f", t, j, 'Przezbrojenie', Przezbrojenie[j, t] >> FileCSV;
	}

	for {j in Produkty, t in Dni union {0}}
		printf "\n;;%d;%s;%s;%3.1f", t, j, 'Gotowosc', Gotowosc[j, t] >> FileCSV;

	for {p in Platformy, s in Samochody, t in Dni}
		printf "\n%s;%s;%d;;%s;%3.1f", p, s, t, 'Transport', Transport[p, s, t] >> FileCSV;

	for {p in Platformy, s in Samochody, t in Dni, j in Produkty}
		printf "\n%s;%s;%d;%s;%s;%3.1f", p, s, t, j, 'Wielkosc_transpD', Wielkosc_transp[p, s, t, j] >> FileCSV;

	for {p in Platformy, s in Samochody, t in Dni, j in Produkty: t-Czas_transp[s] >= 1}
		printf "\n%s;%s;%d;%s;%s;%3.1f", p, s, t, j, 'Wielkosc_transpO', Wielkosc_transp[p, s, t-Czas_transp[s], j] >> FileCSV;

	for {j in Produkty, t in Dni union {0}}
		printf "\n;;%d;%s;%s;%3.1f", t, j, 'Zapas_dostawcy', Zapas_dostawcy[j, t] >> FileCSV;

	for {p in Platformy, j in Produkty, t in Dni union {0}}
		printf "\n%s;;%d;%s;%s;%3.1f", p, t, j, 'Zapas_odbiorcy', Zapas_odbiorcy[p, j, t] >> FileCSV;

	for {o in Okazje}
		printf "\n%s;%s;%d;;%s;%3.1f", Cel_O[o], o, Termin_O[o], 'Transport', Transport_O[o] >> FileCSV;

	for {o in Okazje, j in Produkty}{
		printf "\n%s;%s;%d;%s;%s;%3.1f", Cel_O[o], o, Termin_O[o]-Czas_transp_O[o], j, 'Wielkosc_transpO', Wielkosc_transp_O[o, j] >> FileCSV;
		printf "\n%s;%s;%d;%s;%s;%3.1f", Cel_O[o], o, Termin_O[o], j, 'Wielkosc_transpD', Wielkosc_transp_O[o, j] >> FileCSV;
	}

#======================================================================
end;
